@@ -46,7 +46,7 @@ let cmd =
     `S "SEE ALSO";
     `P "$(b,mv)(1), $(b,scp)(1), $(b,umask)(2), $(b,symlink)(7)" ]
   in
-  Term.(ret (pure cp $ verbose $ recurse $ force $ srcs $ dest)),
+  Term.(ret (const cp $ verbose $ recurse $ force $ srcs $ dest)),
   Term.info "cp" ~version:"1.6.1" ~doc ~man
 
 let () = match Term.eval cmd with `Error _ -> exit 1 | _ -> exit 0
