@@ -40,14 +40,14 @@ let cmd =
         option $(b,--recursive) ($(b,-r) or $(b,-R)).";
     `P "To remove a file whose name starts with a `-', for example
         `-foo', use one of these commands:";
-    `P "rm -- -foo"; `Noblank;
-    `P "rm ./-foo";
+    `Pre "$(mname) -- -foo\n\
+          $(mname) ./-foo";
     `P "$(tname) removes symbolic links, not the files referenced by the
         links.";
     `S "BUGS"; `P "Report bugs to <hehey at example.org>.";
     `S "SEE ALSO"; `P "$(b,rmdir)(1), $(b,unlink)(2)" ]
   in
-  Term.(pure rm $ prompt $ recursive $ files),
+  Term.(const rm $ prompt $ recursive $ files),
   Term.info "rm" ~version:"1.6.1" ~doc ~man
 
 let () = match Term.eval cmd with `Error _ -> exit 1 | _ -> exit 0
