@@ -65,7 +65,7 @@ let cmd =
     `S "SEE ALSO";
     `P "$(b,cat)(1), $(b,head)(1)" ]
   in
-  Term.(pure tail $ lines $ follow $ verb $ pid $ files),
+  Term.(const tail $ lines $ follow $ verb $ pid $ files),
   Term.info "tail" ~version:"1.6.1" ~doc ~man
 
 let () = match Term.eval cmd with `Error _ -> exit 1 | _ -> exit 0
